@@ -1,4 +1,7 @@
 package MooseX::Traits::Pluggable;
+{
+  $MooseX::Traits::Pluggable::VERSION = '0.11';
+}
 
 use namespace::autoclean;
 use Moose::Role;
@@ -6,8 +9,8 @@ use Scalar::Util qw/blessed reftype/;
 use List::MoreUtils 'uniq';
 use Carp;
 use Moose::Util qw/find_meta/;
+use Class::Load qw();
 
-our $VERSION   = '0.10';
 our $AUTHORITY = 'id:RKITOVER';
 
 # stolen from MX::Object::Pluggable
@@ -24,6 +27,11 @@ has '_trait_namespace' => (
   (Moose->VERSION >= 0.84 ) ? (is => 'bare') : (),
 );
 
+has '_traits_behave_like_roles' => (
+  init_arg => undef,
+  (Moose->VERSION >= 0.84 ) ? (is => 'bare') : (),
+);
+
 has _traits => (
   is => 'ro',
   isa => 'ArrayRef[Str]',
@@ -43,27 +51,38 @@ sub _find_trait {
 
     for my $ns (@search_ns) {
         my $full = "${ns}::${base}::${name}";
-        return $full if eval { Class::MOP::load_class($full) };
+        return $full if eval { Class::Load::load_class($full) };
     }
 
     croak "Could not find a class for trait: $name";
 }
 
-sub _transform_trait {
-    my ($class, $name) = @_;
-    my $base;
-    if ($class->can('_trait_namespace')) {
-        $base = $class->_trait_namespace($name);
+my $config_val = sub {
+    my ($class, $attr, @args) = @_;
+
+    my $val;
+
+    if ($class->can($attr)) {
+        $val = $class->$attr(@args);
     }
     else {
-        my $namespace = find_meta($class)->find_attribute_by_name('_trait_namespace');
-        if($namespace->has_default) {
-            $base = $namespace->default;
-            if (ref($base) && reftype($base) eq 'CODE') {
-                $base = $class->$base($name);
+        my $attr_inst = find_meta($class)->find_attribute_by_name($attr);
+        if($attr_inst->has_default) {
+            $val = $attr_inst->default;
+            if (ref($val) && reftype($val) eq 'CODE') {
+                $val = $class->$val(@args);
             }
         }
     }
+
+    return $val;
+};
+
+sub _transform_trait {
+    my ($class, $name) = @_;
+
+    my $base = $config_val->($class, '_trait_namespace', $name);
+
     return $name unless $base;
     return $1 if $name =~ /^[+](.+)$/;
 
@@ -76,7 +95,7 @@ sub _transform_trait {
         }
 
         my $trait = join '::', $ns, $name;
-        return $trait if eval { Class::MOP::load_class($trait) };
+        return $trait if eval { Class::Load::load_class($trait) };
     }
 
     croak "Could not find a class for trait: $name";
@@ -87,7 +106,7 @@ sub _resolve_traits {
 
     return map {
         my $transformed = $class->_transform_trait($_);
-        Class::MOP::load_class($transformed);
+        Class::Load::load_class($transformed);
         $transformed;
     } @traits;
 }
@@ -97,6 +116,26 @@ sub new_with_traits {
     $class->_build_instance_with_traits($class, @_);
 }
 
+my $remove_role_methods_conflicting_with_class = sub {
+    my ($meta, $orig_class, $resolved_traits) = @_;
+
+    my %class_methods;
+    @class_methods{ $orig_class->meta->get_method_list } = ();
+    
+    delete $class_methods{meta};
+
+    my %trait_methods;
+    foreach my $trait (@$resolved_traits) {
+        @trait_methods{ $trait->meta->get_method_list } = ();
+    }
+
+    delete $trait_methods{meta};
+
+    foreach my $class_method (keys %class_methods) {
+        $meta->remove_method($class_method) if exists $trait_methods{$class_method};
+    }
+};
+
 sub _build_instance_with_traits {
     my ($this_class, $class) = (shift, shift);
     my ($hashref, %args, @others) = 0;
@@ -112,7 +151,8 @@ sub _build_instance_with_traits {
 
     if (my $traits = delete $args{traits}) {
         my @traits = ref($traits) ? @$traits : ($traits);
-        if(@traits){
+
+        if (@traits) {
             $args{_traits} = \@traits;
             my @resolved_traits = $this_class->_resolve_traits(@traits);
             $args{_resolved_traits} = \@resolved_traits;
@@ -122,13 +162,19 @@ sub _build_instance_with_traits {
                 roles        => \@resolved_traits,
                 cache        => 1,
             );
+
             # Method attributes in inherited roles may have turned metaclass
             # to lies. CatalystX::Component::Traits related special move
             # to deal with this here.
             $meta = find_meta($meta->name);
 
             $meta->add_method('meta' => sub { $meta });
+            my $orig_class = $class;
             $class = $meta->name;
+
+            if ($config_val->($orig_class, '_traits_behave_like_roles')) {
+                $remove_role_methods_conflicting_with_class->($meta, $orig_class, \@resolved_traits);
+            }
         }
     }
 
@@ -157,6 +203,12 @@ sub apply_traits {
         for my $trait (@resolved_traits){
             $trait->meta->apply($self, rebless_params => $rebless_params);
         }
+
+        my $orig_class = $self->_original_class_name;
+
+        if ($config_val->($orig_class, '_traits_behave_like_roles')) {
+            $remove_role_methods_conflicting_with_class->($self->meta, $orig_class, \@resolved_traits);
+        }
     }
 }
 
@@ -203,6 +255,7 @@ Example:
   use parent 'Class1';
   with 'MooseX::Traits';
   has '+_trait_namespace' => (default => '+Trait');
+  has '+_traits_behave_like_roles' => (default => 1);
 
   package Class2::Trait::Bar;
   use Moose::Role;
@@ -233,6 +286,20 @@ You can search multiple namespaces for traits, for example:
 Will search in the C<class_precedence_list> for C<::Trait::TheTrait>
 and C<::Role::TheTrait> and then for C<ExtraNS::Trait::TheTrait>.
 
+=head1 CORRECT ROLE BEHAVIOR
+
+By default, a method from a role will override a class method, this however is
+not the behavior one expects when applying a L<Moose> role using the normal
+methods.
+
+If you want the behavior to be consistent with L<Moose> L<roles|Moose::Role>,
+then use this configuration attribute in your class:
+
+  has '+_traits_behave_like_roles' => (default => 1);
+
+This may or may not become the default in the future, for now you have to ask
+for it for backward compatibility reasons.
+
 =head1 EXTRA ATTRIBUTES
 
 =head2 _original_class_name
@@ -251,7 +318,7 @@ List of traits applied to the instance resolved to full package names.
 
 =head1 SEE ALSO
 
-L<MooseX::Traits>, L<MooseX::Object::Pluggable>
+L<MooseX::Traits>, L<MooseX::Object::Pluggable>, L<CatalystX::Component::Traits>
 
 =head1 BUGS
 
@@ -261,30 +328,6 @@ L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=MooseX-Traits-Pluggable>.  I
 will be notified, and then you'll automatically be notified of progress on your
 bug as I make changes.
 
-=head1 SUPPORT
-
-More information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=MooseX-Traits-Pluggable>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/MooseX-Traits-Pluggable>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/MooseX-Traits-Pluggable>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/MooseX-Traits-Pluggable/>
-
-=back
-
 =head1 AUTHOR
 
 Rafael Kitover C<< <rkitover@cpan.org> >>
@@ -295,8 +338,7 @@ Tomas Doran, C<< <bobtfish@bobtfish.net> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (c) 2009 - 2010 by the aforementioned
-L</AUTHOR> and L</CONTRIBUTORS>.
+Copyright (c) 2014 by the aforementioned L</AUTHOR> and L</CONTRIBUTORS>.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
